#include <unistd.h>
#include <errno.h>

#define IOT_BUFSIZE 256

static void *mbtk_glfw_iot_stdio(void *tctx)
{
	glfw_display_t *gd = tctx;
	char *bufs = malloc(IOT_BUFSIZE*2); /* can't be static because there may be multiple stdio threads for multiple displays */
	int bi;

	for(bi = 0;; bi = !bi) {
		int len;
		char *buf = bi ? &bufs[IOT_BUFSIZE] : &bufs[0]; /* need to double buffer because we read in the background while the previous is processed by the user */

		len = read(0, buf, IOT_BUFSIZE-1);
		if (len > 0)
			buf[len] = '\0';
		else
			buf[0] = '\0';

		gd->iot_stdio.data = buf;
		gd->iot_stdio.len = len;

		/* wait while we are in disable */
		mbtk_be_semwait(gd->iot_stdio.enable, 1);

		gd->iot_stdio.loaded = 1;
		glfwPostEmptyEvent();

		/* main thread will increase handled to 1 to indicate the event has been processed */
		mbtk_be_semwait(gd->iot_stdio.handled, 1);
	}

	free(bufs);
	return NULL;
}

static void mbtk_glfw_watch_stdin_read(mbtk_display_t *disp, int enable)
{
	glfw_display_t *gd = (glfw_display_t *)disp;

	if (enable) {
		if (!gd->iot_stdio.thread_inited) {
			gd->iot_stdio.handled = mbtk_be_semget(0);
			gd->iot_stdio.enable = mbtk_be_semget(0);
			mbtk_be_thread_create(&gd->iot_stdio.tid, mbtk_glfw_iot_stdio, gd);
			gd->iot_stdio.thread_inited = 1;
		}

		/* enable reporting */
		mbtk_be_semwait(gd->iot_stdio.enable, 0); /* decrease enable to 0 */
		mbtk_be_sempost(gd->iot_stdio.enable); /* increase to 1 to enable */
		gd->iot_stdio.disabled = 0;
	}
	else {
		mbtk_be_semwait(gd->iot_stdio.enable, 0); /* decrease enable to 0 so the read thread is blocked */
		gd->iot_stdio.disabled = 1;
	}
}


/*** fdwatch ***/

typedef struct {
	int fd;
	unsigned for_read:1;
	unsigned for_write:1;
} mbtk_glfw_fdwacth_req_t;

static void *mbtk_glfw_iot_fdwatch(void *tctx)
{
	glfw_display_t *gd = tctx;
	mbtk_watch_fd_t wctx = {0};
	mbtk_event_t dst;

	mbtk_watch_fd_set(&wctx, gd->iot_fdwatch.ctrlr, 1, 0);

	for(;;) {
		long usec = 1000000-1;
		int res;

		res = mbtk_watch_fd_select(&wctx, &dst, &usec, NULL);
		if (res < 0) {
			if (errno == EINTR)
				continue;
			/* something went wrong with an fd, try to blocking-read control to remove stale fds */
			goto read_ctrl;
		}
		if (res == 0)
			continue;

		if (dst.data.fdwatch.fd == gd->iot_fdwatch.ctrlr) {

			mbtk_glfw_fdwacth_req_t req;
			int len;

			read_ctrl:;

			len = read(gd->iot_fdwatch.ctrlr, &req, sizeof(req));
			if (len != sizeof(req))
				mbtk_error("mbtk_glfw_iot_fdwatch(): short read or read error on pipe: %d != %d\n", len, (int)sizeof(req));
			else
				mbtk_watch_fd_set(&wctx, req.fd, req.for_read, req.for_write);

			continue;
		}

		gd->iot_fdwatch.fd = dst.data.fdwatch.fd;
		gd->iot_fdwatch.readable = dst.data.fdwatch.readable;
		gd->iot_fdwatch.writable = dst.data.fdwatch.writable;

		gd->iot_fdwatch.loaded = 1;
		glfwPostEmptyEvent();

		/* main thread will increase handled to 1 to indicate the event has been processed */
		mbtk_be_semwait(gd->iot_fdwatch.handled, 1);
	}
	return NULL;
}

static void mbtk_glfw_watch_fd(mbtk_display_t *disp, int fd, int for_read, int for_write)
{
	glfw_display_t *gd = (glfw_display_t *)disp;
	mbtk_glfw_fdwacth_req_t req;
	int len;

	if (!gd->iot_fdwatch.thread_inited) {
		mbtk_be_pipe(&gd->iot_fdwatch.ctrlr, &gd->iot_fdwatch.ctrlw);
			mbtk_be_thread_create(&gd->iot_fdwatch.tid, mbtk_glfw_iot_fdwatch, gd);
		gd->iot_fdwatch.thread_inited = 1;
		gd->iot_fdwatch.handled = mbtk_be_semget(0);
	}

	req.fd = fd;
	req.for_read = for_read;
	req.for_write = for_write;
	len = write(gd->iot_fdwatch.ctrlw, &req, sizeof(req)); /* lazy: assume atomic write */
	if (len != sizeof(req))
		mbtk_error("mbtk_sdl2_watch_fd(): short write or write error on pipe: %d != %d\n", len, (int)sizeof(req));
}
