#ifndef LIBMBTK_WIDGET_BUTTON_H
#define LIBMBTK_WIDGET_BUTTON_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Button: can be clicked a mouse click release or enter release */
typedef struct mbtk_button_s {
	mbtk_widget_t w;
} mbtk_button_t;

/* Create a new button with no child widget. The caller shall create a widget
   (e.g. a hbox with further widgets) and use mbtk_button_set_child() to put
   install it in the button. Handler is set to hand_psuhb. */
mbtk_button_t *mbtk_button_new(mbtk_button_t *caller_alloced);

/* Create a new button with a label installed in it. Note: always dynamically
   alloced because of the child label widget needs to be alloced. Use
   mbtk_button_new_with_widget() for static allocation. */
mbtk_button_t *mbtk_button_new_with_label(const char *str);

/* Create a new button with a widget installed in it. Can be used with static
   allocation, for example:
   mbtk_button_new_with_widget(&w1, mbtk_label_new(&w2, "foo")); */
mbtk_button_t *mbtk_button_new_with_widget(mbtk_button_t *caller_alloced, mbtk_widget_t *child);


/* A button hosts a single widget (which can be a container, e.g. hbox or vbox).
   If the button already had a child widget, it is unlinked (removed from the
   button) but not freed.
   Returns original child on success (may be NULL). */
mbtk_widget_t *mbtk_button_set_child(mbtk_button_t *parent, mbtk_widget_t *w);


/* Called after the release on a click on the button */
void mbtk_button_callback_click(mbtk_button_t *b, int (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data);


MBTK_WIDGET_INLINES(button)

#endif
