#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/draw.h>
#include <libmbtk/widget/Common.h>
#include <libmbtk/widget/hand_toggle.h>

#include <libmbtk/widget/checkbox.h>


static void mbtk_checkbox_imp_destroy(mbtk_widget_t *w) { }

#define FALLBACK_SIZE 10

static int mbtk_checkbox_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	long l, r, t, b;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	w->hvbox.fr_l = l; w->hvbox.fr_r = r;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r+FALLBACK_SIZE; *my = t+b+FALLBACK_SIZE;

	return 0;
}

static void mbtk_checkbox_fallback_draw(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	wi = l + r + FALLBACK_SIZE; he = t + b + FALLBACK_SIZE;

	style->draw_bg_rect(win, sp, flags, w->hvbox.x, w->hvbox.y, wi, he);
	style->draw_box_frame(win, sp, flags, w->hvbox.x, w->hvbox.y, wi, he);

	/* draw the X mark when activated */
	if (w->flags.activated) {
		long x1 = w->hvbox.x + l, y1 = w->hvbox.y + t;
		long x2 = x1 + FALLBACK_SIZE-r, y2 = y1 + FALLBACK_SIZE - b;

		style->set_fg_color(win, sp, w->flags);
		mbtk_draw_line(win, x1, y1, x2, y2);
		mbtk_draw_line(win, x1, y2, x2, y1);
	}
}

void mbtk_checkbox_fallback_draw_overlay(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	wi = l + r + FALLBACK_SIZE; he = t + b + FALLBACK_SIZE;

	style->draw_box_overlay(win, sp, flags, w->hvbox.x, w->hvbox.y, wi, he);
}



static const mbtk_widget_imp_t mbtk_checkbox_imp = {
	mbtk_checkbox_imp_destroy,

	mbtk_checkbox_fallback_minsize,
	mbtk_checkbox_fallback_draw,
	mbtk_checkbox_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	mbtk_comm_embed1_pre_insert,
	NULL /* mbtk_checkbox_post_insert */
};

mbtk_checkbox_t *mbtk_checkbox_new(mbtk_checkbox_t *caller_alloced)
{
	mbtk_checkbox_t *checkbox;
	mbtk_kw_t kw_checkbox;

	mbtk_kw_cache(kw_checkbox, "checkbox");

	mbtk_create_wt(checkbox, kw_checkbox, &mbtk_checkbox_imp, caller_alloced);
	checkbox->w.hvbox.type = HVBOX_HBOX;
	mbtk_handler_toggle_install(&checkbox->w);
	return checkbox;
}

void mbtk_checkbox_callback_toggle(mbtk_checkbox_t *b, int (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data)
{
	mbtk_kw_t kw_toggled;
	mbtk_kw_cache(kw_toggled, "toggled");
	mbtk_callback_set_noarg(&b->w, kw_toggled, cb, user_data);
}

