#include <stdio.h>
#include <libmbtk/global_typedefs.h>
#include <libmbtk/handler.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>

static const char my_name[] = "hand_pushb";

typedef struct mbtk_pushb_handler_s {
	mbtk_handler_t hdr;
} mbtk_pushb_handler_t;

static mbtk_pushb_handler_t pushb_handler;
static int pushb_handler_inited = 0;

static void pushb_fire_cb(mbtk_widget_t *widget)
{
	mbtk_kw_t kw_clicked;

	mbtk_kw_cache(kw_clicked, "clicked");
	mbtk_callback_noarg(widget, kw_clicked);
}

static void pushb_event(mbtk_widget_t *widget, mbtk_event_t *ev)
{
	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
			if (ev->data.key.sym == MBTK_KS_ENTER) {
				mbtk_widget_set_activated(widget, 1);
				pushb_fire_cb(widget);
			}
			break;
		case MBTK_EV_KEY_RELEASE:
			if (ev->data.key.sym == MBTK_KS_ENTER)
				mbtk_widget_set_activated(widget, 0);
			break;
		case MBTK_EV_MOUSE_PRESS:
			if (ev->data.mouse.button == MBTK_MB_LEFT) {
				mbtk_window_set_focus(widget->win, widget, 1);
				mbtk_widget_set_activated(widget, 1);
				widget->win->origin = widget;
			}
			break;
		case MBTK_EV_MOUSE_RELEASE:
			if (ev->data.mouse.button == MBTK_MB_LEFT) {
				if (widget->flags.activated)
					pushb_fire_cb(widget);
				mbtk_widget_set_activated(widget, 0);
			}
			break;

		case MBTK_EV_FOCUS_ENTER:
			mbtk_widget_set_activated(widget, (widget == widget->win->origin) && mbtk_display_get_mouse_button(widget->win->display, MBTK_MB_LEFT));
			break;
		case MBTK_EV_FOCUS_LEAVE:
			mbtk_widget_set_activated(widget, 0);
			break;

		default:;
	}
}


void mbtk_handler_pushb_install(mbtk_widget_t *widget)
{
	if (widget->handler != NULL)
		mbtk_widget_del_handler(widget);

	if (!pushb_handler_inited) {
		pushb_handler.hdr.name = my_name;
		pushb_handler.hdr.event = pushb_event;
		pushb_handler_inited = 1;
	}
	widget->handler = &pushb_handler.hdr;
}
