#include <stdio.h>
#include <libmbtk/global_typedefs.h>
#include <libmbtk/handler.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>

static const char my_name[] = "hand_pushb";

typedef struct mbtk_toggle_handler_s {
	mbtk_handler_t hdr;
} mbtk_toggle_handler_t;

static mbtk_toggle_handler_t toggle_handler;
static int toggle_handler_inited = 0;

static void toggle_fire_cb(mbtk_widget_t *widget)
{
	mbtk_kw_t kw_toggled;

	mbtk_kw_cache(kw_toggled, "toggled");
	mbtk_callback_noarg(widget, kw_toggled);
}

static void toggle_event(mbtk_widget_t *widget, mbtk_event_t *ev)
{
	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
			if (ev->data.key.sym == MBTK_KS_ENTER) {
				mbtk_widget_set_activated(widget, !widget->flags.activated);
				toggle_fire_cb(widget);
			}
			break;
		case MBTK_EV_MOUSE_PRESS:
			if (ev->data.mouse.button == MBTK_MB_LEFT) {
				mbtk_window_set_focus(widget->win, widget, 1);
				mbtk_widget_set_activated(widget, !widget->flags.activated);
				widget->win->origin = widget;
			}
			break;
		case MBTK_EV_MOUSE_RELEASE:
			if (ev->data.mouse.button == MBTK_MB_LEFT)
				toggle_fire_cb(widget);
			break;

		case MBTK_EV_FOCUS_ENTER:
			if ((widget == widget->win->origin) && mbtk_display_get_mouse_button(widget->win->display, MBTK_MB_LEFT))
				mbtk_widget_set_activated(widget, !widget->flags.activated);
			break;
		case MBTK_EV_FOCUS_LEAVE:
			if (widget == widget->win->origin)
				mbtk_widget_set_activated(widget, !widget->flags.activated);
			break;

		default:;
	}
}


void mbtk_handler_toggle_install(mbtk_widget_t *widget)
{
	if (widget->handler != NULL)
		mbtk_widget_del_handler(widget);

	if (!toggle_handler_inited) {
		toggle_handler.hdr.name = my_name;
		toggle_handler.hdr.event = toggle_event;
		toggle_handler_inited = 1;
	}
	widget->handler = &toggle_handler.hdr;
}
