#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/draw.h>
#include <libmbtk/widget/progbar.h>
#include <libmbtk/widget/Common.h>


static void mbtk_progbar_imp_destroy(mbtk_widget_t *w) { }

#define FILLSIZE 4

static int mbtk_progbar_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	long l, r, t, b;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	w->hvbox.fr_l = l; w->hvbox.fr_r = r;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r+FILLSIZE; *my = t+b+FILLSIZE;

	return 0;
}

static void mbtk_progbar_fallback_draw(mbtk_widget_t *w)
{
	mbtk_progbar_t *prb = (mbtk_progbar_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, bx1, by1, bwi, bhe;
	long x1 = w->hvbox.x, y1 = w->hvbox.y, wi = w->hvbox.w, he = w->hvbox.h-1;
	double pos;
	mbtk_kw_t kw_hilight_color;

	/* clamp and normalize between 0 and 1 */
	if (prb->max > prb->min) {
		pos = prb->curr;
		if (pos < prb->min) pos = prb->min;
		if (pos > prb->max) pos = prb->max;
		pos = (pos  - prb->min) / (prb->max - prb->min);
	}
	else
		pos = 0;


	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	flags.activated = 0;
	style->draw_bg_rect(win, sp, flags, x1, y1, wi, he);

	/* tune dimensions with the frame for the bar */
	bx1 = x1+l; by1 = y1+t;
	bwi = wi-(l+r);
	bhe = he-(t+b);

	mbtk_kw_cache(kw_hilight_color, "hilight.color");

	if (mbtk_widget_draw_set_style_color(w, kw_hilight_color) != 0)
		style->set_fg_color(win, sp, w->flags);
	mbtk_draw_fillrect(win, bx1, by1, bx1 + ((double)bwi * pos) - 1, by1+bhe-1);

	flags.activated = 1;
	style->draw_box_frame(win, sp, flags, x1, y1, wi, he);
}


static const mbtk_widget_imp_t mbtk_progbar_imp = {
	mbtk_progbar_imp_destroy,

	mbtk_progbar_fallback_minsize,
	mbtk_progbar_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	mbtk_comm_embed1_pre_insert,
	NULL /* mbtk_progbar_post_insert */
};

mbtk_progbar_t *mbtk_progbar_new(mbtk_progbar_t *caller_alloced)
{
	mbtk_progbar_t *progbar;
	mbtk_kw_t kw_progbar;

	mbtk_kw_cache(kw_progbar, "progbar");

	mbtk_create_wt(progbar, kw_progbar, &mbtk_progbar_imp, caller_alloced);
	progbar->min = 0.0;
	progbar->max = 1.0;
	progbar->curr = 0.0;
	progbar->w.hvbox.type = HVBOX_HBOX; /* don't count padding be part of the widget */

	return progbar;
}
