#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/cursor.h>
#include "../be_init.h"

/* for testing mpm $ colors */
#include <libmbtk/style_f3d.h>
#include <libmbtk/mpm.h>

static mbtk_window_t *aux_win[10];

const char *test_mpm[] = {
	"! MPM1",
	"24 7 3",
	"o $frame_dark.color",
	"+ $frame_dark.color alpha * 0.5",
	". #00000000",
	"o....o.oooo..ooooo.o...o",
	"oo..oo.o..+o...o...o..o.",
	"o.oo.o.o..+o...o...o.o..",
	"o.++.o.oooo....o...oo+..",
	"o....o.o..+o...o...o.o+.",
	"o....o.o..+o...o...o..o+",
	"o....o.oooo....o...o...o",
};


unsigned char *gen_px_alpha(int w, int h)
{
	unsigned char *res = malloc(w*h*4), *s;
	int x, y;
	double tot = (double)w+(double)h;
	
	s = res;
	for(y = 0; y < h; y++) {
		for(x = 0; x < w; x++, s+=4) {
			s[0] = 255;
			s[1] = 255;
			s[2] = 0;
			s[3] = (double)(x+y)/tot * 255;
		}
	}

	return res;
}

unsigned char *gen_px_solid(int w, int h)
{
	unsigned char *res = malloc(w*h*3), *s;
	int x, y;
	double tot = (double)w+(double)h;
	
	s = res;
	for(y = 0; y < h; y++) {
		for(x = 0; x < w; x++, s+=3) {
			s[0] = 255;
			s[1] = (double)(x+y)/tot * 255;
			s[2] = 0;
		}
	}

	return res;
}

static mbtk_pixmap_t txt_alpha_p, txt_solid_p, txt_solid_3x3_p, txt_mpm_p;
static void *ck_text = NULL, *ck_text2 = NULL;
static mbtk_px_id_t txt_alpha, txt_solid, txt_solid_3x3, txt_mpm;
static char **font;
static int stay;

static void redraw(mbtk_window_t *win)
{
	mbtk_display_t *disp = win->display;

	mbtk_draw_start(win);

	mbtk_draw_color(win, 255, 255, 255, 255);
	mbtk_draw_fillrect(win, 0, 0, 200, 150);

	mbtk_draw_color(win, 0, 128, 255, 128);
	mbtk_draw_fillrect(win, 0, 0, 2, 2);
	mbtk_draw_fillrect(win, 4, 4, 6, 6);
	mbtk_draw_fillrect(win, 8, 8, 10, 10);
	mbtk_draw_pixmap(win, 12, 12, txt_solid_3x3);

	mbtk_draw_color(win, 0, 255, 0, 128);
	mbtk_draw_fillrect(win, 60, 40, 160, 50);
	mbtk_draw_color(win, 255, 0, 255, 128);
	mbtk_draw_line(win, 20, 10, 130, 111);

	mbtk_draw_color(win, 255, 0, 64, 255);
	mbtk_draw_hline(win, 0, 1, 2);
	mbtk_draw_vline(win, 1, 0, 2);
	mbtk_draw_line(win, 4, 4, 6, 6);
	mbtk_draw_line(win, 8, 10, 10, 8);

	mbtk_draw_pixmap(win, 65, 35, txt_alpha);
	mbtk_draw_pixmap(win, 100, 35, txt_solid);
	mbtk_draw_pixmap(win, 18, 18, txt_mpm);

	{ /* text */
		int w, asc, desc;
		mbtk_font_id_t fid;
		char *msg = "Mj^-.A.";

		fid = mbtk_resolve_font(disp, 1, font);
		if (fid.l == -1)
			fprintf(stderr, "ERROR: failed to open preferred font.\n");
		mbtk_draw_set_font(win, fid);

		mbtk_draw_query_text(win, &w, &asc, &desc, msg, strlen(msg));
		mbtk_draw_fillrect(win, 10+w, 50, 10+w+5, 70);
		mbtk_draw_fillrect(win, 5, 50, 10, 70);
		mbtk_draw_hline(win, 10, 50, 10+w);

		mbtk_draw_color(win, 255, 0, 0, 128);
		mbtk_draw_text(win, 10, 50, msg, strlen(msg), &ck_text);
		mbtk_draw_text(win, 10, 50+asc*2, backend_name, strlen(backend_name), &ck_text2);
		mbtk_draw_color(win, 0, 255, 0, 128);
	}

	mbtk_draw_flush(win);
}

static void printev(mbtk_event_t *ev)
{
	const char *name = mbtk_event_type_name(ev->type);

	printf(" [%s] ", (const char *)ev->win->user_data);

	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
		case MBTK_EV_KEY_RELEASE:
			printf("kev %s: %d %d %x %lu '%c'\n", name, ev->data.key.scancode, ev->data.key.sym, ev->data.key.mods, ev->data.key.edit, (char)ev->data.key.edit);
			break;
		case MBTK_EV_MOUSE_PRESS:
		case MBTK_EV_MOUSE_RELEASE:
			printf("mev %s: %ld %ld [%d]\n", name, ev->data.mouse.x, ev->data.mouse.y, ev->data.mouse.button);
			break;
		case MBTK_EV_MOUSE_MOTION:
		case MBTK_EV_MOUSE_SCROLL:
			printf("mev %s: %ld %ld\n", name, ev->data.mouse.x, ev->data.mouse.y);
			break;
		case MBTK_EV_WIN_EXPOSE:
			printf("win expose: %ld %ld (%ld %ld)\n", ev->data.win.x, ev->data.win.y, ev->data.win.w, ev->data.win.h);
			redraw(ev->win);
			break;
		case MBTK_EV_WIN_RESIZE:
			printf("win RESIZE: %ld %ld (at %ld %ld)\n", ev->data.win.w, ev->data.win.h, ev->data.win.x, ev->data.win.y);
			break;
		case MBTK_EV_WIN_MOVE:
			printf("win MOVE: %ld %ld (size %ld %ld)\n", ev->data.win.x, ev->data.win.y, ev->data.win.w, ev->data.win.h);
			break;
		case MBTK_EV_WIN_CLOSE:
			printf("win CLOSE\n");
			stay = 0;
			break;
		case MBTK_EV_WIN_ENTER:
			printf("win mouse enter\n");
			break;
		case MBTK_EV_WIN_LEAVE:
			printf("win mouse leave\n");
			break;
		case MBTK_EV_RSTDIN:
			printf("stdin read: %ld/%s\n", (long)ev->data.rstdin.len, ev->data.rstdin.data);
			break;
		case MBTK_EV_FDWATCH:
			printf("fdwatch %d r%d w%d c%d\n", ev->data.fdwatch.fd, ev->data.fdwatch.readable, ev->data.fdwatch.writable, ev->data.fdwatch.closed);
			if (ev->data.fdwatch.readable) {
				int len;
				char buf[256];
				len = read(ev->data.fdwatch.fd, buf, sizeof(buf)-1);
				if (len > 0) {
					buf[len] = '\0';
					printf(" %d '%s'\n", len, buf);
				}
				else {
					printf(" <CLOSE> %d\n", len);
					close(ev->data.fdwatch.fd);
					mbtk_watch_fd(ev->win->display, ev->data.fdwatch.fd, 0, 0);
				}
			}
			break;
		case MBTK_EV_FOCUS_ENTER:
			printf("focus enter; ERROR: shouldn't get this\n");
			break;
		case MBTK_EV_FOCUS_LEAVE:
			printf("focus leave; ERROR: shouldn't get this\n");
			break;
		case MBTK_EV_max:
			printf("max; ERROR: shouldn't get this\n");
			break;
	}
	fflush(stdout);

	ev->win = NULL; /* to detect if it is not set in a subsequent call */
}

/* cycle through named mouse cursors */
static void cursor_named(mbtk_display_t *disp)
{
	static mbtk_mocu_named_t cn = MBTK_MOCU_IBEAM;
	static mbtk_mocu_t *last = NULL;

	if (last != NULL)
		mbtk_mocu_free_cursor(disp, last);

	if (cn > MBTK_MOCU_VARROW)
		cn = MBTK_MOCU_ARROW;

	last = mbtk_mocu_create_named(disp, cn++);
	mbtk_mocu_set_cursor(disp, last);
}

/* Draw semi-random pixmap for cursor */
static void cursor_pixmap(mbtk_display_t *disp)
{
	static mbtk_mocu_t *last = NULL;
	static int pat;
	unsigned char px[16*16/8], mk[16*16/8];
	int n;

	if (last != NULL)
		mbtk_mocu_free_cursor(disp, last);

	for(n = 0; n < 16*16/8; n++) {
		switch(pat % 2) {
			case 0:
				switch(n % 8) {
					case 0:
					case 1:
						px[n] = 0;
						mk[n] = 0xff;
						break;
					default:
						px[n] = rand();
						mk[n] = rand();
				}
				break;
			case 1:
				px[n] = n;
				mk[n] = 0xff;
				break;
		}
	}
	last = mbtk_mocu_create_pixmap(disp, 16, 16, px, mk, 0, 0);
	mbtk_mocu_set_cursor(disp, last);
	pat++;
}

static void retitle(mbtk_window_t *win)
{
	char title[32];
	static short int cnt = 0;
	sprintf(title, "title #%d", cnt++);
	mbtk_window_set_title(win, title);
}

static void open_win(mbtk_display_t *disp, int type)
{
	if (type == 0) {
		int n;
		for(n = 1; n < 10; n++) {
			if (aux_win[n] != NULL) {
				mbtk_window_close(aux_win[n]);
				aux_win[n] = NULL;
			}
		}
		return;
	}

	if (aux_win[type] != NULL) {
		mbtk_window_close(aux_win[type]);
		aux_win[type] = NULL;
		return;
	}


	aux_win[type] = mbtk_window_new(disp, type, "miniboxtk test: child window", -1, -1, 100, 200);
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	unsigned char *raw;
	int fd;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp); /* for testing mpm $ colors */

	raw = gen_px_alpha(32, 32);
	mbtk_draw_import_pixmap(&disp, &txt_alpha_p, raw, 32, 32, 1);
	memset(raw, 0xff, 32*32*3);
	free(raw);

	raw = gen_px_solid(32, 32);
	mbtk_draw_import_pixmap(&disp, &txt_solid_p, raw, 32, 32, 0);
	memset(raw, 0xff, 32*32*3);
	free(raw);

	raw = gen_px_solid(3, 3);
	mbtk_draw_import_pixmap(&disp, &txt_solid_3x3_p, raw, 3, 3, 0);
	memset(raw, 0xff, 3*3*3);
	free(raw);

	if (mbtk_draw_import_mpm(&disp, disp.style, disp.styleprop, &txt_mpm_p, NULL, test_mpm) != 0) {
		fprintf(stderr, "Failed to load test_mpm\n");
		return 1;
	}


	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk test", -1, -1, 200, 150);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}
	mbtk_window_set_min_size(win, 100, 50);
	win->user_data = "top window";

	txt_alpha = mbtk_draw_bind_pixmap(win, &txt_alpha_p);
	txt_solid = mbtk_draw_bind_pixmap(win, &txt_solid_p);
	txt_solid_3x3 = mbtk_draw_bind_pixmap(win, &txt_solid_3x3_p);
	txt_mpm = mbtk_draw_bind_pixmap(win, &txt_mpm_p);

	mbtk_watch_stdin_read(&disp, 1);
	fd = open("P", O_RDONLY);
	if (fd > 0)
		mbtk_watch_fd(&disp, fd, 1, 0);

	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_be_iterate_(&disp, &ev, 10.0, 1)) {
			printev(&ev);
			if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;
				if (ev.data.key.sym == 'e')
					mbtk_watch_stdin_read(&disp, 1);
				if (ev.data.key.sym == 'd')
					mbtk_watch_stdin_read(&disp, 0);
				if ((ev.data.key.sym == 'h') && (aux_win[1] != NULL))
					mbtk_window_hide(aux_win[1]);
				if ((ev.data.key.sym == 's') && (aux_win[1] != NULL))
					mbtk_window_show(aux_win[1]);
				if ((ev.data.key.sym == 'r') && (aux_win[1] != NULL))
					mbtk_window_raise(aux_win[1]);
				if (ev.data.key.sym == 'c')
					cursor_named(&disp);
				if (ev.data.key.sym == 'x')
					cursor_pixmap(&disp);
				if (ev.data.key.sym == 't')
					retitle(win);
				if (ev.data.key.sym == '0')
					open_win(&disp, 0);
				if (ev.data.key.sym == '1')
					open_win(&disp, 1);
				if (ev.data.key.sym == '2')
					open_win(&disp, 2);
				if (ev.data.key.sym == '3')
					open_win(&disp, 3);
				if (ev.data.key.sym == '4')
					open_win(&disp, 4);
				if (ev.data.key.sym == '5')
					open_win(&disp, 5);
				if (ev.data.key.sym == '6')
					open_win(&disp, 6);
				if (ev.data.key.sym == '7')
					open_win(&disp, 7);
				if (ev.data.key.sym == 'i')
					mbtk_window_set_size(win, 300, 200);
				if (ev.data.key.sym == 't')
					printf("time: %.4f\n", mbtk_get_time_el(&disp));
				if (ev.data.key.sym == 'o')
					mbtk_clipboard_set(&disp, "Hello world from miniboxtk!");
				if (ev.data.key.sym == 'p') {
					char *tmp = mbtk_clipboard_get(&disp);
					printf("CLIP: '%s'\n", tmp);
					mbtk_clipboard_free(&disp, tmp);
				}
			}
		}
	}

	mbtk_draw_free_text(win, &ck_text);
	mbtk_draw_free_text(win, &ck_text2);
	mbtk_draw_unbind_pixmap(win, txt_alpha);
	mbtk_draw_unbind_pixmap(win, txt_solid);
	mbtk_draw_unbind_pixmap(win, txt_mpm);
	mbtk_draw_free_pixmap(&disp, &txt_alpha_p);
	mbtk_draw_free_pixmap(&disp, &txt_solid_p);
	mbtk_draw_free_pixmap(&disp, &txt_mpm_p);
	mbtk_display_uninit(&be, &disp);
	mbtk_be_uninit(&be);
}
